'#########################################################################################
' Toolbar Control example
'#########################################################################################

#DIM ALL
#COMPILE EXE
#INCLUDE ONCE "ToolbarCtrl.inc"
#RESOURCE "EX_TOOLBAR_03.PBR"

GLOBAL   ghInstance              AS DWORD    ' handle of the application instance

'--------------------------- [ Command Identifiers ] ---------------------------

%IDM_ARROWLEFT                               = 28000
%IDM_ARROWRIGHT                              = 28001
%IDM_COPYCLIPBOARD                           = 28002
%IDM_COPYTOFOLDER                            = 28003
%IDM_CUTCLIPBOARD                            = 28004
%IDM_DELETE                                  = 28005
%IDM_FAVORITES                               = 28006
%IDM_FOLDERCLOSED                            = 28007
%IDM_FOLDEROPEN                              = 28008
%IDM_FOLDEROPTIONS                           = 28009
%IDM_FOLDERS                                 = 28010

'--------------------------- [ Control Identifiers ] ---------------------------

%IDC_TOOLBAR                                 = 101
%IDC_COMBO                                   = 102


'-------------------------------------------------------------------------------
'
' PROCEDURE: WinMain
' PURPOSE:   Program entry point, calls initialization function, processes
'            message loop.
'
'-------------------------------------------------------------------------------

FUNCTION WinMain _
   ( _
   BYVAL hInstance      AS DWORD, _       ' handle of current instance
   BYVAL hPrevInstance  AS DWORD, _       ' handle of previous instance(not used in Win32)
   BYVAL pszCmdLine     AS ASCIIZ PTR, _  ' address of command line
   BYVAL nCmdShow       AS LONG _         ' show state of window
   ) AS LONG

   LOCAL szClassName       AS ASCIIZ * %MAX_PATH         ' class name
   LOCAL twcx              AS WNDCLASSEX                 ' class information
   LOCAL tmsg              AS tagMsg                     ' message information
   LOCAL ticc              AS INIT_COMMON_CONTROLSEX     ' specifies common control classes to register
   LOCAL hWnd              AS DWORD                      ' handle of main window

   ' Save the handle of the application instance
   ghInstance = hInstance

   ' Register the Form1 window
   szClassName        = "EX_TOOLBAR_03"
   twcx.cbSize        = SIZEOF(twcx)                              ' size of WNDCLASSEX structure
   twcx.style         = %CS_DBLCLKS                               ' class styles
   twcx.lpfnWndProc   = CODEPTR(Form1_WndProc)                    ' address of window procedure used by class
   twcx.cbClsExtra    = 0                                         ' extra class bytes
   twcx.cbWndExtra    = 0                                         ' extra window bytes
   twcx.hInstance     = ghInstance                                ' instance of the process that is registering the window
   twcx.hIcon         = LoadIcon(%NULL, BYVAL %IDI_APPLICATION)   ' handle of class icon
   twcx.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)       ' handle of class cursor
   twcx.hbrBackground = %COLOR_BTNFACE + 1                        ' brush used to fill background of window's client area
   twcx.lpszMenuName  = %NULL                                     ' resource identifier of the class menu
   twcx.lpszClassName = VARPTR(szClassName)                       ' class name
   twcx.hIconSm       = %NULL                                     ' handle of small icon shown in caption/system Taskbar
   IF ISFALSE RegisterClassEx(twcx) THEN
      FUNCTION = %TRUE
      EXIT FUNCTION
   END IF

   ' Load the common controls library and
   ' specify the classes to register.
   ticc.dwSize = SIZEOF(ticc)
   ticc.dwICC  = %ICC_BAR_CLASSES
   InitCommonControlsEx ticc

   ' Create the Form1 window
   hWnd = CreateWindowEx(%WS_EX_WINDOWEDGE, _                                          ' extended styles
                         szClassName, _                                                ' class name
                         "Toolbar Demo", _                                             ' caption
                         %WS_OVERLAPPEDWINDOW OR %WS_VISIBLE, _                        ' window styles
                         %CW_USEDEFAULT, 0, _                                          ' left, top
                         %CW_USEDEFAULT, 0, _                                          ' width, height
                         %NULL, %NULL, _                                               ' handle of owner, menu handle
                         ghInstance, BYVAL %NULL)                                      ' handle of instance, creation parameters

   ' If window could not be created, return "failure"
   IF ISFALSE hWnd THEN
      FUNCTION = %FALSE
      EXIT FUNCTION
   END IF

   ' Make the window visible; update its client area
   ShowWindow hWnd, nCmdShow
   UpdateWindow hWnd

   ' Main message loop of program.
   ' Acquire and dispatch messages until a WM_QUIT message is received.
   WHILE ISTRUE GetMessage(tmsg, BYVAL %NULL, 0, 0)
      IF ISFALSE IsDialogMessage(hWnd, tmsg) THEN
         TranslateMessage tmsg
         DispatchMessage tmsg
      END IF
   WEND

   FUNCTION = tmsg.wParam

END FUNCTION

'-------------------------------------------------------------------------------
'
' PROCEDURE: Form1_WndProc
' PURPOSE:   Processes messages for the Form1 window.
'
'-------------------------------------------------------------------------------

FUNCTION Form1_WndProc _
   ( _
   BYVAL hWnd     AS DWORD, _ ' window handle
   BYVAL uMsg     AS DWORD, _ ' type of message
   BYVAL wParam   AS DWORD, _ ' first message parameter
   BYVAL lParam   AS LONG _   ' second message parameter
   ) EXPORT AS LONG

   LOCAL sBtnText       AS STRING               ' toolbar button caption
   LOCAL ttbb           AS TBBUTTON             ' specifies or receives the attributes of a toolbar button
   LOCAL ttbab          AS TBADDBITMAP          ' specifies the images to add to a toolbar
   LOCAL ptnmhdr        AS NMHDR PTR            ' information about a notification message
   LOCAL ptttdi         AS NMTTDISPINFO PTR     ' tooltip notification message information
   LOCAL pttbb          AS TBBUTTON PTR         ' address of array of toolbar button info
   LOCAL hFont          AS DWORD                ' handle of font used by form
   LOCAL lMsgResult     AS LONG                 ' value returned to message after message is processed
   LOCAL hImageList     AS DWORD                ' handle of imagelist control
   LOCAL hImage         AS DWORD                ' handle of bitmap
   LOCAL tbn            AS TBNOTIFY PTR         ' for toolbar notification messages
   LOCAL rc             AS RECT                 ' bounding rectangle for a specified toolbar button
   LOCAL szItem         AS ASCIIZ * 256         ' working variable
   LOCAL hToolbar       AS DWORD                ' handle of the tooolbar
   LOCAL hCombo         AS DWORD                ' handle of the combobox

   SELECT CASE uMsg

      CASE %WM_COMMAND

         ' -------------------------------------------------------
         ' Messages from the Toolbar are handled here.
         ' ------------------------------------------------------

         SELECT CASE LOWRD(wParam)

            CASE %IDCANCEL
               IF HIWRD(wParam) = %BN_CLICKED THEN
                  SendMessage hWnd, %WM_CLOSE, wParam, lParam
                  FUNCTION = %FALSE
                  EXIT FUNCTION
               END IF

            CASE %IDM_ARROWLEFT
               MSGBOX "Arrow left button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_ARROWRIGHT
               MSGBOX "Arrow right button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_COPYCLIPBOARD
               MSGBOX "Copy to clipboard button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_COPYTOFOLDER
               MSGBOX "Copy to folder button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_CUTCLIPBOARD
               MSGBOX "Cut to clipboard button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_DELETE
               MSGBOX "Delete button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_FAVORITES
               MSGBOX "Favorites button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_FOLDERCLOSED
               MSGBOX "Folder closed button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_FOLDEROPEN
               MSGBOX "Folder open button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_FOLDEROPTIONS
               MSGBOX "Folder options button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

            CASE %IDM_FOLDERS
               MSGBOX "Folders button clicked"
               FUNCTION = %FALSE
               EXIT FUNCTION

         END SELECT

      CASE %WM_NOTIFY

         ' -------------------------------------------------------
         ' Notification messages are handled here.
         ' The TTN_GETDISPINFO message is sent by a ToolTip control
         ' to retrieve information needed to display a ToolTip window.
         ' ------------------------------------------------------
         ptnmhdr = lParam
         tbn = lParam

         SELECT CASE @ptnmhdr.code
            CASE %TTN_GETDISPINFO
               ptttdi        = lParam
               @ptttdi.hinst = ghInstance
               SELECT CASE @ptttdi.hdr.hwndFrom
                  ' The TB_GETTOOLTIPS message retrieves the handle to the ToolTip control,
                  ' if any, associated with the toolbar.
                  CASE SendMessage(GetDlgItem(hWnd, %IDC_TOOLBAR), %TB_GETTOOLTIPS, 0, 0)
                     @ptttdi.lpszText = @ptttdi.hdr.idFrom
                     EXIT FUNCTION
               END SELECT
         END SELECT

      CASE %WM_SYSCOLORCHANGE
         ' Forward this message to common controls so that they will
         ' be properly updated when the user changes the color settings.
         SendMessage GetDlgItem(hWnd, %IDC_TOOLBAR), %WM_SYSCOLORCHANGE, wParam, lParam

      CASE %WM_DESTROY
         ' Destroy the image lists
         hToolbar = GetDlgItem(hWnd, %IDC_TOOLBAR)
         ImageList_Destroy SendMessage(hToolbar, %TB_SETIMAGELIST, 0, %NULL)
         ImageList_Destroy SendMessage(hToolbar, %TB_SETDISABLEDIMAGELIST, 0, %NULL)
         ImageList_Destroy SendMessage(hToolbar, %TB_SETHOTIMAGELIST, 0, %NULL)
         ' End the application
         PostQuitMessage 0
         FUNCTION = %FALSE
         EXIT FUNCTION

      CASE %WM_SIZE
         IF wParam <> %SIZE_MINIMIZED THEN
            ' Update the size and position of the Toolbar control
            SendMessage GetDlgItem(hWnd, %IDC_TOOLBAR), %TB_AUTOSIZE, 0, 0
         END IF
         FUNCTION = %FALSE
         EXIT FUNCTION

      CASE %WM_CREATE
         ' Create font used by container
         hFont = GetStockObject(%DEFAULT_GUI_FONT)

         ' Create the toolbar control
         hToolbar = CreateWindowEx(%NULL, _                                              ' extended styles
                                    "ToolbarWindow32", _                                  ' class name
                                    "", _                                                 ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR _                         ' window styles
                                    %CCS_TOP OR %TBSTYLE_TOOLTIPS OR _
                                    %TBSTYLE_FLAT, _                                      ' class styles
                                    0, 0, _                                               ' left, top
                                    345, 116, _                                           ' width, height
                                    hWnd, %IDC_TOOLBAR, _                                 ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hToolbar, %WM_SETFONT, hFont, %TRUE

         ' Set the extended class styles for the control
         SendMessage hToolbar, %TB_SETEXTENDEDSTYLE, 0, %TBSTYLE_EX_DRAWDDARROWS

         ' Set the imagelist used with default images
         hImageList = ImageList_Create(24, 24, %ILC_MASK OR %ILC_COLOR24, 1, 0)
         hImage = LoadImage(ghInstance, "TBNOR", %IMAGE_BITMAP, 0, 0, %LR_DEFAULTSIZE)
         ImageList_AddMasked hImageList, hImage, RGB(255,0,255)
         DeleteObject hImage
         SendMessage hToolbar, %TB_SETIMAGELIST, 0, hImageList

         ' Set the imagelist used with disabled images
         hImageList = ImageList_Create(24, 24, %ILC_MASK OR %ILC_COLOR24, 1, 0)
         hImage = LoadImage(ghInstance, "TBDIS", %IMAGE_BITMAP, 0, 0, %LR_DEFAULTSIZE)
         ImageList_AddMasked hImageList, hImage, RGB(255,0,255)
         DeleteObject hImage
         SendMessage hToolbar, %TB_SETDISABLEDIMAGELIST, 0, hImageList

         ' Set the imagelist used with hot images
         hImageList = ImageList_Create(24, 24, %ILC_MASK OR %ILC_COLOR24, 1, 0)
         hImage = LoadImage(ghInstance, "TBHOT", %IMAGE_BITMAP, 0, 0, %LR_DEFAULTSIZE)
         ImageList_AddMasked hImageList, hImage, RGB(255,0,255)
         DeleteObject hImage
         SendMessage hToolbar, %TB_SETHOTIMAGELIST, 0, hImageList

         ' Allocate memory for the button info array
         pttbb = HeapAlloc(GetProcessHeap(), %HEAP_ZERO_MEMORY, 13 * SIZEOF(ttbb))
         IF ISTRUE pttbb THEN
            ' Send the TB_BUTTONSTRUCTSIZE message, for backward compatibility
            SendMessage hToolbar, %TB_BUTTONSTRUCTSIZE, SIZEOF(ttbb), 0
            ' Set the size of the bitmaps
            SendMessage hToolbar, %TB_SETBITMAPSIZE, 0, MAKLNG(24, 24)
            ' Add buttons to the toolbar
            @pttbb[0].iBitmap    = 0
            @pttbb[0].idCommand  = %IDM_ARROWLEFT
            @pttbb[0].fsState    = %TBSTATE_ENABLED
            @pttbb[0].fsStyle    = %BTNS_CHECKGROUP
            @pttbb[0].dwData     = 0
            @pttbb[0].iString    = -1

            @pttbb[1].iBitmap    = 1
            @pttbb[1].idCommand  = %IDM_ARROWRIGHT
            @pttbb[1].fsState    = %TBSTATE_ENABLED
            @pttbb[1].fsStyle    = %BTNS_CHECKGROUP
            @pttbb[1].dwData     = 0
            @pttbb[1].iString    = -1

            @pttbb[2].iBitmap    = 2
            @pttbb[2].idCommand  = %IDM_COPYCLIPBOARD
            @pttbb[2].fsState    = %TBSTATE_ENABLED
            @pttbb[2].fsStyle    = %BTNS_BUTTON
            @pttbb[2].dwData     = 0
            @pttbb[2].iString    = -1

            @pttbb[3].iBitmap    = 0
            @pttbb[3].idCommand  = 0
            @pttbb[3].fsState    = %TBSTATE_ENABLED
            @pttbb[3].fsStyle    = %BTNS_SEP
            @pttbb[3].dwData     = 0
            @pttbb[3].iString    = -1

            @pttbb[4].iBitmap    = 3
            @pttbb[4].idCommand  = %IDM_COPYTOFOLDER
            @pttbb[4].fsState    = %TBSTATE_ENABLED
            @pttbb[4].fsStyle    = %BTNS_BUTTON
            @pttbb[4].dwData     = 0
            @pttbb[4].iString    = -1

            @pttbb[5].iBitmap    = 4
            @pttbb[5].idCommand  = %IDM_CUTCLIPBOARD
            @pttbb[5].fsState    = %TBSTATE_ENABLED
            @pttbb[5].fsStyle    = %BTNS_CHECK
            @pttbb[5].dwData     = 0
            @pttbb[5].iString    = -1

            @pttbb[6].iBitmap    = 5
            @pttbb[6].idCommand  = %IDM_DELETE
            @pttbb[6].fsState    = %TBSTATE_ENABLED
            @pttbb[6].fsStyle    = %BTNS_BUTTON
            @pttbb[6].dwData     = 0
            @pttbb[6].iString    = -1

            @pttbb[7].iBitmap    = 6
            @pttbb[7].idCommand  = %IDM_FAVORITES
            @pttbb[7].fsState    = %TBSTATE_ENABLED
            @pttbb[7].fsStyle    = %BTNS_BUTTON
            @pttbb[7].dwData     = 0
            @pttbb[7].iString    = -1

            @pttbb[8].iBitmap    = 210   ' Make room for the combobox
            @pttbb[8].idCommand  = 0
            @pttbb[8].fsState    = %TBSTATE_ENABLED
            @pttbb[8].fsStyle    = %BTNS_SEP
            @pttbb[8].dwData     = 0
            @pttbb[8].iString    = -1

            @pttbb[9].iBitmap    = 7
            @pttbb[9].idCommand  = %IDM_FOLDERCLOSED
            @pttbb[9].fsState    = %TBSTATE_ENABLED
            @pttbb[9].fsStyle    = %BTNS_BUTTON
            @pttbb[9].dwData     = 0
            @pttbb[9].iString    = -1

            @pttbb[10].iBitmap    = 8
            @pttbb[10].idCommand  = %IDM_FOLDEROPEN
            @pttbb[10].fsState    = %TBSTATE_ENABLED
            @pttbb[10].fsStyle    = %BTNS_BUTTON
            @pttbb[10].dwData     = 0
            @pttbb[10].iString    = -1

            @pttbb[11].iBitmap    = 9
            @pttbb[11].idCommand  = %IDM_FOLDEROPTIONS
            @pttbb[11].fsState    = %TBSTATE_ENABLED
            @pttbb[11].fsStyle    = %BTNS_BUTTON
            @pttbb[11].dwData     = 0
            @pttbb[11].iString    = -1

            @pttbb[12].iBitmap   = 10
            @pttbb[12].idCommand = %IDM_FOLDERS
            @pttbb[12].fsState   = %TBSTATE_ENABLED
            @pttbb[12].fsStyle   = %BTNS_BUTTON
            @pttbb[12].dwData    = 0
            @pttbb[12].iString   = -1

            SendMessage hToolbar, %TB_ADDBUTTONS, 13, BYVAL pttbb
            ' Free memory that was allocated for the button info
            HeapFree GetProcessHeap(), 0, BYVAL pttbb
            ' Update the size of the toolbar
            SendMessage hToolbar, %TB_AUTOSIZE, 0, 0
         END IF

         ' Create the Combo1 combobox
         hCombo = CreateWindowEx(%WS_EX_CLIENTEDGE, _                                  ' extended styles
                                    "Combobox", _                                         ' class name
                                    "", _                                                 ' caption
                                    %WS_CHILD OR %WS_VISIBLE OR %WS_VSCROLL OR _          ' window styles
                                    %WS_BORDER OR %WS_TABSTOP OR _
                                    %CBS_DROPDOWN OR %CBS_HASSTRINGS OR %CBS_SORT, _      ' class styles
                                    228, 5, _                                            ' left, top
                                    200, 105, _                                           ' width, height
                                    hWnd, %IDC_COMBO, _                                   ' handle of parent, control ID
                                    ghInstance, BYVAL %NULL)                              ' handle of instance, creation parameters
         SendMessage hCombo, %WM_SETFONT, hFont, %TRUE
         SetParent hCombo, hToolbar

         ' Add the initial items
         ' Turn off repainting
         SendMessage hCombo, %WM_SETREDRAW, %FALSE, 0
         szItem = "Item 5"
         SendMessage hCombo, %CB_ADDSTRING, 0, BYVAL VARPTR(szItem)
         szItem = "Item 4"
         SendMessage hCombo, %CB_ADDSTRING, 0, BYVAL VARPTR(szItem)
         szItem = "Item 3"
         SendMessage hCombo, %CB_ADDSTRING, 0, BYVAL VARPTR(szItem)
         szItem = "Item 2"
         SendMessage hCombo, %CB_ADDSTRING, 0, BYVAL VARPTR(szItem)
         szItem = "Item 1"
         SendMessage hCombo, %CB_ADDSTRING, 0, BYVAL VARPTR(szItem)
         szItem = "Item 0"
         SendMessage hCombo, %CB_ADDSTRING, 0, BYVAL VARPTR(szItem)
         ' Turn on repainting
         SendMessage hCombo, %WM_SETREDRAW, %TRUE, 0
         InvalidateRect hCombo, BYVAL %NULL, %TRUE
         UpdateWindow hCombo

         FUNCTION = %FALSE
         EXIT FUNCTION
   END SELECT

   FUNCTION = DefWindowProc(hWnd, uMsg, wParam, lParam)

END FUNCTION
'=========================================================================================
